@@ -10,6 +10,7 @@ Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
 Copyright 2019      Daniel Lublin <daniel@lublin.se>
 Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2019      Jade Auer <jade@trashwitch.dev>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -46,13 +47,15 @@ init_parse_options(int argc, char **argv)
 
    opt.quality = 75;
    opt.overwrite = 0;
+   opt.line_style = LineSolid;
+   opt.line_width = 1;
 
    /* Parse the cmdline args */
    scrot_parse_option_array(argc, argv);
 }
 
-static int
-parse_option_required_number(char *str)
+int
+options_parse_required_number(char *str)
 {
    char *end = NULL;
    int ret = 0;
@@ -73,12 +76,74 @@ parse_option_required_number(char *str)
    return ret;
 }
 
+static void
+options_parse_line(char *optarg)
+{
+   enum {STYLE = 0, WIDTH };
+
+   char *const token[] = {
+      [STYLE] = "style",
+      [WIDTH] = "width",
+      NULL
+   };
+
+   char *subopts = optarg;
+   char *value = NULL;
+
+   while (*subopts != '\0') {
+      switch(getsubopt(&subopts, token, &value)) {
+
+         case STYLE:
+
+            if (value == NULL) {
+               fprintf(stderr, "Missing value for "
+                     "suboption '%s'\n", token[STYLE]);
+               exit(EXIT_FAILURE);
+            }
+
+            if (!strncmp(value, "dash", 4))
+               opt.line_style = LineOnOffDash;
+            else if (!strncmp(value, "solid", 5))
+               opt.line_style = LineSolid;
+            else {
+               fprintf(stderr, "Unknown value for "
+                     "suboption '%s': %s\n", token[STYLE], value);
+               exit(EXIT_FAILURE);
+            }
+            break;
+
+         case WIDTH:
+
+            if (value == NULL) {
+               fprintf(stderr, "Missing value for "
+                     "suboption '%s'\n", token[WIDTH]);
+               exit(EXIT_FAILURE);
+            }
+
+            opt.line_width = options_parse_required_number(value);
+
+            if (opt.line_width <= 0 || opt.line_width > 8){
+               fprintf(stderr, "Value of the range (1..8) for "
+                     "suboption '%s': %d\n", token[WIDTH], opt.line_width);
+               exit(EXIT_FAILURE);
+            }
+            break;
+
+         default:
+            fprintf(stderr, "No match found for token: '%s'\n", value);
+            exit(EXIT_FAILURE);
+            break;
+      }
+
+   } /* while */
+}
 
 
 static void
 scrot_parse_option_array(int argc, char **argv)
 {
-   static char stropts[] = "a:ofpbcd:e:hmq:st:uv+:z";
+   static char stropts[] = "a:ofpbcd:e:hmq:st:uv+:zn:l:";
+
    static struct option lopts[] = {
       /* actions */
       {"help", 0, 0, 'h'},                  /* okay */
@@ -100,6 +165,8 @@ scrot_parse_option_array(int argc, char **argv)
       {"exec", 1, 0, 'e'},
       {"debug-level", 1, 0, '+'},
       {"autoselect", required_argument, 0, 'a'},
+      {"note", required_argument, 0, 'n'},
+      {"line", required_argument, 0, 'l'},
       {0, 0, 0, 0}
    };
    int optch = 0, cmdx = 0;
@@ -122,7 +189,7 @@ scrot_parse_option_array(int argc, char **argv)
            opt.border = 1;
            break;
         case 'd':
-           opt.delay = parse_option_required_number(optarg);
+           opt.delay = options_parse_required_number(optarg);
            break;
         case 'e':
            opt.exec = gib_estrdup(optarg);
@@ -131,7 +198,7 @@ scrot_parse_option_array(int argc, char **argv)
            opt.multidisp = 1;
            break;
         case 'q':
-           opt.quality = parse_option_required_number(optarg);
+           opt.quality = options_parse_required_number(optarg);
            break;
         case 's':
            opt.select = 1;
@@ -140,7 +207,7 @@ scrot_parse_option_array(int argc, char **argv)
            opt.focused = 1;
            break;
         case '+':
-           opt.debug_level = parse_option_required_number(optarg);
+           opt.debug_level = options_parse_required_number(optarg);
            break;
         case 'c':
            opt.countdown = 1;
@@ -163,6 +230,12 @@ scrot_parse_option_array(int argc, char **argv)
         case 'a':
            options_parse_autoselect(optarg);
            break;
+        case 'n':
+           options_parse_note(optarg);
+           break;
+        case 'l':
+           options_parse_line(optarg);
+           break;
         case '?':
            exit(EXIT_FAILURE);
         default:
@@ -234,9 +307,9 @@ options_parse_autoselect(char *optarg)
 
    if (strchr(optarg, ',')) /* geometry dimensions must be in format x,y,w,h   */
    {
-     dimensions[i++] = parse_option_required_number(strtok(optarg, tokdelim));
+     dimensions[i++] = options_parse_required_number(strtok(optarg, tokdelim));
      while ((tok = strtok(NULL, tokdelim)) )
-        dimensions[i++] = parse_option_required_number(tok);
+        dimensions[i++] = options_parse_required_number(tok);
      opt.autoselect=1;
      opt.autoselect_x=dimensions[0];
      opt.autoselect_y=dimensions[1];
@@ -253,12 +326,12 @@ options_parse_thumbnail(char *optarg)
    if (strchr(optarg, 'x')) /* We want to specify the geometry */
    {
      tok = strtok(optarg, "x");
-     opt.thumb_width = parse_option_required_number(tok);
+     opt.thumb_width = options_parse_required_number(tok);
      tok = strtok(NULL, "x");
      if (tok)
      {
-       opt.thumb_width = parse_option_required_number(optarg);
-       opt.thumb_height = parse_option_required_number(tok);
+       opt.thumb_width = options_parse_required_number(optarg);
+       opt.thumb_height = options_parse_required_number(tok);
 
        if (opt.thumb_width < 0)
          opt.thumb_width = 1;
@@ -273,7 +346,7 @@ options_parse_thumbnail(char *optarg)
    }
    else
    {
-     opt.thumb = parse_option_required_number(optarg);
+     opt.thumb = options_parse_required_number(optarg);
      if (opt.thumb < 1)
        opt.thumb = 1;
      else if (opt.thumb > 100)
@@ -281,6 +354,20 @@ options_parse_thumbnail(char *optarg)
    }
 }
 
+void options_parse_note(char *optarg)
+{
+   opt.note = gib_estrdup(optarg);
+
+   if (opt.note == NULL) return;
+
+   if (opt.note[0] == '\0') {
+      fprintf(stderr, "Required arguments for --note.");
+      exit(EXIT_FAILURE);
+   }
+
+   scrot_note_new(opt.note);
+}
+
 void
 show_version(void)
 {
@@ -328,6 +415,10 @@ show_usage(void)
            "  -p, --pointer             Capture the mouse pointer.\n"
            "  -f, --freeze              Freeze the screen when the selection is used: --select\n"
            "  -o, --overwrite           By default " SCROT_PACKAGE " does not overwrite the files, use this option to allow it.\n"
+           "  -l, --line                Indicates the style of the line when the selection is used: --select\n"
+           "                            See SELECTION STYLE\n"
+           "  -n, --note                Draw a text note.\n"
+           "                            See NOTE FORMAT\n"
 
            "\n" "  SPECIAL STRINGS\n"
            "  Both the --exec and filename parameters can take format specifiers\n"
@@ -338,6 +429,7 @@ show_usage(void)
            "  The second kind are internal to " SCROT_PACKAGE
            "  and are prefixed by '$'\n"
            "  The following specifiers are recognised:\n"
+           "                  $a hostname\n"
            "                  $f image path/filename (ignored when used in the filename)\n"
            "                  $m thumbnail path/filename\n"
            "                  $n image name (ignored when used in the filename)\n"
@@ -352,6 +444,23 @@ show_usage(void)
            " '%%Y-%%m-%%d_$wx$h_scrot.png' -e 'mv $f ~/images/shots/'\n"
            "          Creates a file called something like 2000-10-30_2560x1024_scrot.png\n"
            "          and moves it to your images directory.\n" "\n"
+           "\n" "  SELECTION STYLE\n"
+           "  When using --select you can indicate the style of the line with --line.\n"
+           "  The following specifiers are recognised:\n"
+           "                  style=(solid,dash),width=(range 1 to 8)\n"
+           "  The default style are:\n"
+           "                  style=solid,width=1\n"
+           "  Example:\n" "          " SCROT_PACKAGE
+           " --line style=dash,width=3 --select\n\n"
+           "\n" "  NOTE FORMAT\n"
+           "  The following specifiers are recognised for the option --note\n"
+           "                  -f 'FontName/size'\n"
+           "                  -t 'text'\n"
+           "                  -x position (optional)\n"
+           "                  -y position (optional)\n"
+           "                  -c color(RGBA) (optional)\n"
+           "  Example:\n" "          " SCROT_PACKAGE
+           " --note \"-f '/usr/share/fonts/TTF/DroidSans-Bold/40' -x 10 -y 20 -c 255,0,0,255 -t 'Hi'\"\n\n"
            "This program is free software see the file COPYING for licensing info.\n"
            "Copyright Tom Gilbert 2000\n"
            "Email bugs to <scrot_sucks@linuxbrit.co.uk>\n");
